#pragma once
#include <Arduino.h>
#include <display.h>
#include <SimpleSerialProtocol.h>
#include <passwords.h>
#include <fingersensor.h>

#define SERIAL_PACKET_BUFFER_SIZE 25

namespace Connections {
    void init();
    void handle();
    void sendPacket(char *id, char *type, char *data);
    void onSerialPacket(Packet packet);

    long lastPacketTransmit = 0;
    Packet serialPackets[SERIAL_PACKET_BUFFER_SIZE];
    SimpleSerialProtocol device;

    Packet *name_packet;
}

void Connections::init() {
    Display::drawStatus("Init serial: ...");

    for (int i = 0; i < SERIAL_PACKET_BUFFER_SIZE; i++) {
        serialPackets[i].id = "--";
        serialPackets[i].type = "--";
        serialPackets[i].data = "--";
    }

    Connections::device.init(Serial, 9600);
    Display::drawStatus("Init serial: ok!");

    Serial.print("!***!");
    delay(750);
}

void Connections::handle() {
    device.receivePackets(Connections::onSerialPacket);

    for (int i = 0; i < SERIAL_PACKET_BUFFER_SIZE; i++) {
        if (!(strcmp(serialPackets[i].id, "--") == 0)) {
            if ((millis() - lastPacketTransmit) > 500) {
                device.sendPacket(serialPackets[i].id, serialPackets[i].type, serialPackets[i].data);
            
                serialPackets[i].id = "--";
                serialPackets[i].type = "--";
                serialPackets[i].data = "--";

                lastPacketTransmit = millis();

                break;
            }  
        }
    }
}

void Connections::sendPacket(char *id, char *type, char *data) {
    for (int i = 0; i < SERIAL_PACKET_BUFFER_SIZE; i++) {
        if (strcmp(serialPackets[i].id, "--") == 0) {
            serialPackets[i].id = id;
            serialPackets[i].type = type;
            serialPackets[i].data = data;

            break;
        }
    }
}

void Connections::onSerialPacket(Packet packet) {
    if (strcmp(packet.id, "test") == 0) {
        Connections::sendPacket("received", "received", "received");
    }

    if (strcmp(packet.id, "passwords") == 0 && strcmp(packet.type, "available") == 0) {
        char* passwords = (char*)Passwords::available();
        Connections::sendPacket("passwords", "available_passwords", passwords);
    }

    if (strcmp(packet.id, "fingerprint") == 0 && strcmp(packet.type, "enroll") == 0) {
        int id = atoi(packet.data);
        bool status = FingerPrintSensor::enroll(id);

       if (status) {
            
            Connections::sendPacket("fingerprint", "auth_status", "valid");
        } else {
            Connections::sendPacket("fingerprint", "auth_status", "invalid");
        }
        // char* status = (char*)char(enroll_status);
        Connections::sendPacket("fingerprint", "enroll_status", "");
    }

    if (strcmp(packet.id, "fingerprint") == 0 && strcmp(packet.type, "auth") == 0) {
        int status = FingerPrintSensor::authenticate();
        if (status) {
            Connections::sendPacket("fingerprint", "auth_status", "valid");
            Display::drawStatus("Ready!");
            FingerPrintSensor::color(2);

        } else {
            Connections::sendPacket("fingerprint", "auth_status", "invalid");
            Display::drawStatus("Not authenticated!");
            FingerPrintSensor::color(1);
        }
    }

    if (strcmp(packet.id, "passwords") == 0 && strcmp(packet.type, "available") == 0) {
        const char * data = Passwords::available();
        Connections::sendPacket("passwords", "available_passwords", (char*)data);
    }

    if (strcmp(packet.id, "new_password") == 0) {
        Passwords::store(packet.type, packet.data);
    }

    if (strcmp(packet.id, "passwords") == 0 && strcmp(packet.type, "new_available") == 0) {
        Passwords::setAvailable(packet.data);
    }

    if (strcmp(packet.id, "passwords") == 0 && strcmp(packet.type, "load") == 0) {
        const char* password = Passwords::get(packet.data);
        // const char* password = "ds";
        Connections::sendPacket("passwords", "password", (char*)password);
    }
}