#pragma once

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_RESET 0  // GPIO0
#define SCREEN_ADDRESS 0x3C
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

namespace Display
{
    Adafruit_SSD1306 _display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, SCREEN_RESET);

    void init();
    void drawText(int x, int y, char* text);
    void drawRectangle(int x, int y, int w, int h);
    void drawStatus(char* status);
}


void Display::init() {
    // Init screen bus
    if(!_display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
        return;
    }

    // Init screen buffer
    _display.display();
    _display.clearDisplay();

    Display::drawRectangle(0, 0, 128, 32);
    Display::drawText(24, 3, "Securitas Mk 1");
    Display::drawText(8, 16, "Made by Nick Leeman");

    delay(750);
}

void Display::drawStatus(char* status) {
    _display.clearDisplay();
    Display::drawRectangle(0, 0, 128, 32);
    Display::drawText(3, 3, "Securitas Mk 1");
    Display::drawText(95, 3, "21:28");
    Display::drawText(3, 20, status);
}

void Display::drawText(int x, int y, char* text) {
    // Normal 1:1 pixel scale
    _display.setTextSize(1);

    // Draw white text
    _display.setTextColor(SSD1306_WHITE);
    
    // Start at top-left corner
    _display.setCursor(x, y);

    // Use full 256 char 'Code Page 437' font
    _display.cp437(true);

    // Write text
    _display.printf(text); 

    // Update buffer
    _display.display();     
}

void Display::drawRectangle(int x, int y, int w, int h) {
    _display.drawRect(x, y, w, h, SSD1306_WHITE);
}