#pragma once

#include <SoftwareSerial.h>
#include <Adafruit_Fingerprint.h>

namespace FingerPrintSensor
{
    SoftwareSerial fingerSensorSerial(12, 14);
    Adafruit_Fingerprint finger = Adafruit_Fingerprint(&fingerSensorSerial);

    bool authenticated = false;

    void init();
    bool enroll(int id);
    bool authenticate();
    void color(int type);
} 

void FingerPrintSensor::init() {
    Display::drawStatus("Init fingerprint: ...");

    // set the data rate for the sensor serial port
    finger.begin(57600);
    delay(50);

    if (!finger.verifyPassword()) {
        Display::drawStatus("Init fingerprint: err!");
        return;
    }

    finger.getParameters();

    Display::drawStatus("Init fingerprint: ok!");
    delay(750);
}

bool FingerPrintSensor::enroll(int id) {
    Display::drawStatus("Enrolling process");
    delay(1000);

    int status = -1;
    
    while (status != FINGERPRINT_OK) {
        status = finger.getImage();
        
        switch (status) {
            case FINGERPRINT_OK:
                Display::drawStatus("Image captured!");
                delay(1000);
            break;

            case FINGERPRINT_NOFINGER:
                Display::drawStatus("Place finger...");
                delay(250);
            break;

            case FINGERPRINT_PACKETRECIEVEERR:
                Display::drawStatus("Comms error...");
                delay(1000);
            break;

            case FINGERPRINT_IMAGEFAIL:
                Display::drawStatus("Image error!");
                delay(1000);
            break;

            default:
                Display::drawStatus("Unknown error!");
                delay(1000);
            break;
        }
    }

    status = finger.image2Tz(1);
    switch (status) {
        case FINGERPRINT_OK:
            Display::drawStatus("Image Converted!");
            delay(1000);
        break;
        
        case FINGERPRINT_IMAGEMESS:
            Display::drawStatus("Bad image quality!");
            delay(1000);
        return false;
        
        case FINGERPRINT_PACKETRECIEVEERR:
            Display::drawStatus("Comms error...");
            delay(1000);
        return false;
        
        case FINGERPRINT_FEATUREFAIL:
            Display::drawStatus("No features!");
            delay(1000);
        return false;
        
        case FINGERPRINT_INVALIDIMAGE:
            Display::drawStatus("Invalid features!");
            delay(1000);
        return false;
        
        default:
            Display::drawStatus("Unknown error!");
            delay(1000);
        return false;
    }

    status = 0;
    while (status != FINGERPRINT_NOFINGER) {
        status = finger.getImage();
    }

    
    status = -1;
    Display::drawStatus("Place finger again!");
    delay(1000);
    
    while (status != FINGERPRINT_OK) {
        status = finger.getImage();
        
        switch (status) {
            case FINGERPRINT_OK:
                Display::drawStatus("Image captured!");
                delay(1000);
            break;
            
            case FINGERPRINT_NOFINGER:
                Display::drawStatus("Place finger...");
                delay(250);
            break;
            
            case FINGERPRINT_PACKETRECIEVEERR:
                Display::drawStatus("Comms error...");
                delay(1000);
            break;
            
            case FINGERPRINT_IMAGEFAIL:
                Display::drawStatus("Image error!");
                delay(1000);
            break;
            
            default:
                Display::drawStatus("Unknown error!");
                delay(1000);
            break;
        }
    }

    status = finger.image2Tz(2);
    switch (status) {
        case FINGERPRINT_OK:
            Display::drawStatus("Image converted!");
            delay(1000);
        break;
        
        case FINGERPRINT_IMAGEMESS:
            Display::drawStatus("Bad image quality!");
            delay(1000);
        return false;
        
        case FINGERPRINT_PACKETRECIEVEERR:
            Display::drawStatus("Comms error...");
            delay(1000);
        return false;
        
        case FINGERPRINT_FEATUREFAIL:
            Display::drawStatus("No features!");
            delay(1000);
        return false;
        
        case FINGERPRINT_INVALIDIMAGE:
            Display::drawStatus("Invalid features!");
            delay(1000);
        return false;
        
        default:
            Display::drawStatus("Unknown error!");
            delay(1000);
        return false;
    }

    Display::drawStatus("Creating model...");
    delay(1000);

    status = finger.createModel();
    if (status == FINGERPRINT_OK) {
        Display::drawStatus("Enroll match!");
        delay(1000);
    } 
    
    else if (status == FINGERPRINT_PACKETRECIEVEERR) {
        Display::drawStatus("Comms error...");
        delay(1000);
        return false;
    } 
    
    else if (status == FINGERPRINT_ENROLLMISMATCH) {
        Display::drawStatus("Finger mismatch!");
        delay(1000);
        return false;
    } 
    
    else {
        Display::drawStatus("Unknown error!");
        delay(1000);
        return false;
    }

    Serial.print("ID "); Serial.println(id);
    status = finger.storeModel(id);
    if (status == FINGERPRINT_OK) {
        Display::drawStatus("Enroll success!");
        delay(1000);
    } 
    
    else if (status == FINGERPRINT_PACKETRECIEVEERR) {
        Display::drawStatus("Comms error...");
        delay(1000);
        return false;
    } 
    
    else if (status == FINGERPRINT_BADLOCATION) {
        Display::drawStatus("Invalid id!");
        delay(1000);
        return false;
    } 
    
    else if (status == FINGERPRINT_FLASHERR) {
        Display::drawStatus("Invalid flash!");
        delay(1000);
        return false;
    } 
    
    else {
        Display::drawStatus("Unknown error!");
        delay(1000);
        return false;
    }

    return true;
}

bool FingerPrintSensor::authenticate() {
    uint8_t status = finger.getImage();

    while (status != FINGERPRINT_OK)
    {
        status = finger.getImage();
        switch (status) {
            case FINGERPRINT_OK:
                Display::drawStatus("Finger scanned!");
                delay(1000);
            break;
            
            case FINGERPRINT_NOFINGER:
                Display::drawStatus("Place finger...");
                delay(1000);
            break;

            case FINGERPRINT_PACKETRECIEVEERR:
                Display::drawStatus("Comms error...");
                delay(1000);
            return false;

            case FINGERPRINT_IMAGEFAIL:
                Display::drawStatus("Image error!");
                delay(1000);
            return false;

            default:
                Display::drawStatus("Unknown error!");
                delay(1000);
            return false;
        }
    }
    
    

  // OK success!

    status = finger.image2Tz();
    switch (status) {
        case FINGERPRINT_OK:
            Display::drawStatus("Image converted!");
            delay(1000);
        break;
        
        case FINGERPRINT_IMAGEMESS:
            Display::drawStatus("Bad image quality!");
            delay(1000);
        return false;
        
        case FINGERPRINT_PACKETRECIEVEERR:
            Display::drawStatus("Comms error...");
            delay(1000);
        return false;
        
        case FINGERPRINT_FEATUREFAIL:
            Display::drawStatus("Invalid features!");
            delay(1000);
        return false;
        
        case FINGERPRINT_INVALIDIMAGE:
            Display::drawStatus("Invalid features!");
            delay(1000);
        return false;
        
        default:
            Display::drawStatus("Unknown error!");
            delay(1000);
        return false;
    }

    // OK converted!
    status = finger.fingerSearch();
    if (status == FINGERPRINT_OK) {
        Display::drawStatus("Fingerprint valid!");
        authenticated = true;
        delay(1000);
        return true;
    } 
    
    else if (status == FINGERPRINT_PACKETRECIEVEERR) {
        Display::drawStatus("Comms error!");
        delay(1000);
        return false;
    } 
    
    else if (status == FINGERPRINT_NOTFOUND) {
        Display::drawStatus("No match found!");
        delay(1000);
        return false;
    } 
    
    else {
        Display::drawStatus("Unknown error!");
        delay(1000);
        return false;
    }
}

void FingerPrintSensor::color(int type) {
    switch (type)
    {
        case 0:
            finger.LEDcontrol(FINGERPRINT_LED_OFF, 0, FINGERPRINT_LED_RED);
        break;

        case 1:
            finger.LEDcontrol(FINGERPRINT_LED_ON, 0, FINGERPRINT_LED_RED);
        break;

        case 2:
            finger.LEDcontrol(FINGERPRINT_LED_ON, 0, FINGERPRINT_LED_BLUE);
        break;
    }
}
