#pragma once
#include <display.h>
#include <KeyValueEEPROM.h>
#include <fingersensor.h>

namespace Passwords
{
    void init();
    const char* available();
    void setAvailable(char * passwords);
    void store(char* name, char* password);
    const char* get(char* name);
} 

void Passwords::init() {
    Display::drawStatus("Init Passwords...");
    KeyValueEEPROM.begin();

    if(!KeyValueEEPROM.exists("stored-passwords")) {
        KeyValueEEPROM.set<char*>("stored-passwords", "");
    }

    delay(750);
}

const char* Passwords::available() {
    if (FingerPrintSensor::authenticated) {
        char * key = "stored-passwords";
        return KeyValueEEPROM.get<const char*>(key);
    } else {
        return "-";
    }
}

void Passwords::setAvailable(char * passwords) {
    if (FingerPrintSensor::authenticated) {
        KeyValueEEPROM.set("stored-passwords", passwords);
    }
}

void Passwords::store(char* name, char* password) {
    if (FingerPrintSensor::authenticated) {
        KeyValueEEPROM.set(name, password);
    }
}

const char* Passwords::get(char* name) {
    if (FingerPrintSensor::authenticated) {
        return KeyValueEEPROM.get<const char*>(name, "--");
    }
    
    return "-";
}